"""
Text Generation Using Keywords
Keytotext converts a list of Keywords to sentences and larger texts
"""


__version__ = "1.5.1"

from .pipeline import pipeline
from .trainer import trainer
from .make_dataset import make_dataset
